package com.alihealth.d2d.provtest.controller;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.alihealth.d2d.provtest.domain.DomesticDrugProductionApplyInfo;
import com.alihealth.d2d.provtest.domain.DomesticPharmaBaseInfo;
import com.alihealth.d2d.provtest.domain.ReceiveDataLog;
import com.alihealth.d2d.provtest.entity.*;
import com.alihealth.d2d.provtest.enums.BuzStatusEnum;
import com.alihealth.d2d.provtest.enums.EventTypeEnum;
import com.alihealth.d2d.provtest.enums.SubTypeEnum;
import com.alihealth.d2d.provtest.service.*;
import com.alihealth.d2d.provtest.utils.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.security.*;
import java.security.interfaces.RSAPrivateKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @ClassName ProvTestServerController
 * @Desc 省局对接示例代码  服务端  接收端
 * @Author houduo.wk
 * @Date 2020/3/13
 **/
@RestController
@RequestMapping("/prov/test")
public class ProvTestServerController {

    Logger logger = LoggerFactory.getLogger(getClass());
    /**
     * 证书地址
     */
    private static final String PFX_FILE_PATH = "D:\\ynyp\\药品追溯\\d2d\\yunnan.pfx";
    /**
     * 证书密码
     */
    private static final String PFX_PASSWORD = "12345678";
    /**
     * 加密算法
     */
    private static final String algorithm = "RSA";


    @Autowired
    private IDomesticPharmaBaseInfoService domesticPharmaBaseInfoService;

    @Autowired
    private XmlParserService xmlParserService;

    @Autowired
    private IReceiveDataLogService receiveDataLogService;

    @Autowired
    private IDomesticDrugProductionApplyInfoService domesticDrugProductionApplyInfoService;

    @Autowired
    private EventProcessingService processingService;

    @RequestMapping("/receive")
    public @ResponseBody
    Map<String,Object> receive(HttpServletRequest request) throws IOException {
        RequestWrapper requestWrapper = new RequestWrapper(request);
        String body = requestWrapper.getBodyString();
        System.out.println(body);
        Map<String,Object> paramsTemp = JSON.parseObject(body);
        System.out.println(paramsTemp);
        Map<String,Object> params = JSON.parseObject(paramsTemp.get("req").toString());
        logger.info("ProvTestController.receive.in.params:{}",params);
        try{
            //传输流水号0
            String processId = String.valueOf(params.get("process_id"));
            //事件编号
            String eventId = String.valueOf(params.get("event_id"));
            //主数据类型
            String eventType = String.valueOf(params.get("event_type"));
            //主数据 包含子类型sub_type和数据data
            String event = String.valueOf(params.get("event"));
            //时间戳 YYMMddHHmmssSSS
            String timeStamp = String.valueOf(params.get("timestamp"));
            //版本号
            String ver = String.valueOf(params.get("ver"));

            logger.info("processId:{}|eventId:{}|eventType:{}|event:{}|timeStamp:{}|ver:{}"
                    ,processId,eventId,eventType,event,timeStamp,ver);

            EventPojo eventPojo = JSON.parseObject(event, EventPojo.class);
            //获取私钥
            String privateKey = getPrivateKey();

            logger.info("privateKey:{}",privateKey);


            String de = testDecrypt(privateKey,eventPojo.getData());
            String xml = StreamUtil.getUnStreamContent(de);
            logger.info("解密之后的文件内容:{}", xml);

            //保存数据日志
            ReceiveDataLog log=new ReceiveDataLog();
            log.setEventId(eventId);
            log.setProcessId(processId);
            log.setEventType(eventType);
            log.setEventTypeName(EventTypeEnum.getNameByType(eventType));
            log.setSubType(eventPojo.getSubType());
            log.setSubTypeName(SubTypeEnum.getNameByType(eventPojo.getSubType()));
            log.setData(eventPojo.getData());
            log.setVer(ver);
            receiveDataLogService.save(log);

            //解析xml
           // List<DomesticPharmaBaseInfo> list= XmlStringParser.parseVaccineManufacturers(xml,eventPojo.getSubType());

            //解析并保存数据
//            XmlStringParser.parseXml(xml, "1011", data -> {
//                domesticPharmaBaseInfoService.save((DomesticPharmaBaseInfo) data);
//            });

//            List<DomesticPharmaBaseInfo> producers = xmlParserService.parseXml1(
//                    xml,
//                    DomesticPharmaBaseInfo.class
//            );
            //解析并处理数据
            processingService.processEvent(eventPojo, xml,eventId);

            return ResponseBuilder.success();
        }catch (Exception e){
            logger.error("{}",e);
            return ResponseBuilder.fail(BuzStatusEnum.SYS_ERR);
        }

    }


    /**
     * 私钥解密
     * @param key
     * @param data
     * @return
     * @throws NoSuchAlgorithmException
     * @throws InvalidKeyException
     * @throws NoSuchPaddingException
     * @throws InvalidKeySpecException
     * @throws BadPaddingException
     * @throws IllegalBlockSizeException
     * @throws IOException
     */
    public static String testDecrypt(String key,String data) throws NoSuchAlgorithmException, InvalidKeyException, NoSuchPaddingException, InvalidKeySpecException, IllegalBlockSizeException, BadPaddingException, IOException{
        byte[] decode = java.util.Base64.getDecoder().decode(key);
        RSAPrivateKey priKey = (RSAPrivateKey) KeyFactory.getInstance(algorithm).generatePrivate(new PKCS8EncodedKeySpec(decode));
        //RSA解密
        Cipher ci = Cipher.getInstance(algorithm);
        ci.init(Cipher.DECRYPT_MODE, priKey);


        byte[] bytes = java.util.Base64.getDecoder().decode(data);
        int inputLen = bytes.length;
        int offLen = 0;
        int i = 0;
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        while(inputLen - offLen > 0){
            byte[] cache;
            if(inputLen - offLen > 128){
                cache = ci.doFinal(bytes,offLen,128);
            }else{
                cache = ci.doFinal(bytes,offLen,inputLen - offLen);
            }
            byteArrayOutputStream.write(cache);
            i++;
            offLen = 128 * i;

        }
        byteArrayOutputStream.close();
        byte[] byteArray = byteArrayOutputStream.toByteArray();
        return new String(byteArray);
    }

    /**
     * 获取私钥
     * @return
     * @throws Exception
     */
    private static String getPrivateKey() throws Exception {
        KeyStore keyStore = KeyStoreUtil.loadKetStore(PFX_FILE_PATH,PFX_PASSWORD);
        return Base64.encode(KeyStoreUtil.getPrivateKey(keyStore, PFX_PASSWORD).getEncoded());
    }

    /**
     * 获取公钥
     * @return
     * @throws Exception
     */
    private static String getPublicKey() throws Exception {
        KeyStore keyStore = KeyStoreUtil.loadKetStore(PFX_FILE_PATH,PFX_PASSWORD);
        PublicKey publicKey = KeyStoreUtil.getPublicKey(keyStore);
        return Base64.encode(publicKey.getEncoded());
    }
}
